/**
 * Inserts an item into an array sorted by priority. If two items have the same priority,
 * the item will be inserted later will be placed earlier in the array.
 * @param arr modified by inserting item.
 * @param item
 */
export declare function insertPrioritySorted<T extends {
    priority: number;
}>(arr: T[], item: T): T[];
/**
 * Performs a binary search of a given array, returning the index of the first item
 * for which `partition` returns true. Returns the -1 if there are no items in `arr`
 * such that `partition(item)` is true.
 * @param arr
 * @param partition should return true while less than the partition point.
 */
export declare function binaryFindPartition<T>(arr: readonly T[], partition: (item: T) => boolean): number;
/**
 * Removes an item from the array if the array exists and the item is included
 * within it.
 * @param arr
 * @param item
 */
export declare function removeIfPresent<T>(arr: T[] | undefined, item: T): void;
/**
 * Remove items in an array which match a predicate.
 * @param arr
 * @param predicate
 */
export declare function removeIf<T>(arr: T[], predicate: (item: T) => boolean): void;
/**
 * Filters out duplicate values from the given iterable.
 * @param arr
 */
export declare function unique<T>(arr: Iterable<T> | undefined): T[];
/**
 * Filters out duplicate values from the given array with a custom equals check.
 * @param arr
 */
export declare function uniqueByEquals<T extends {
    equals(other: T): boolean;
}>(arr: readonly T[] | undefined): T[];
/**
 * Ensures the given item is an array.
 * @param item
 */
export declare function toArray<T>(item: T | readonly T[] | undefined): T[];
export declare function zip<T extends Iterable<any>[]>(...args: T): Iterable<{
    [K in keyof T]: T[K] extends Iterable<infer U> ? U : T[K];
}>;
export declare function filterMap<T, U>(arr: readonly T[], fn: (item: T, index: number) => U | undefined): U[];
export declare function flatMap<T, U>(arr: readonly T[], fn: (item: T) => U | readonly U[]): U[];
